@@ -2,48 +2,52 @@ PAIR=""
 INPUT_FILE=""
 OUTPUT_FILE=""
 
-function message
+message ()
 {
   echo "USAGE: $(basename $0) [-d datadir] [-f format] [-u] <translation> [in [out]]"
   echo " -d datadir       directory of linguistic data"
-  echo " -f format        one of: txt (default), html, rtf, odt, docx, wxml, xlsx"
+  echo " -f format        one of: txt (default), html, rtf, odt, docx, wxml, xlsx, pptx"
+  echo " -a               display ambiguity"
   echo " -u               don't display marks '*' for unknown words" 
+  echo " -m memory.tmx    use a translation memory to recycle translations"
+  echo " -o translation   translation direction using the translation memory,"
+  echo "                  by default 'translation' is used instead"
   echo " translation      typically, LANG1-LANG2, but see modes.xml in language data"
   echo " in               input file (stdin by default)"
   echo " out              output file (stdout by default)"
   exit 1;
 }
 
-function locale_utf8
+locale_utf8 ()
 {
   export LC_CTYPE=$(locale -a|grep -i "utf[.]*8"|head -1);
-  if [[ LC_CTYPE == "" ]]
+  if [[ LC_CTYPE = "" ]]
   then echo "Error: Install an UTF-8 locale in your system";
        exit 1;
   fi
 }
 
-function test_zip
+test_zip ()
 {
- if [[ $(which zip) == "" ]]
+ if [[ $(which zip) = "" ]]
   then echo "Error: Install 'zip' command in your system";
        exit 1;
   fi
   
-  if [[ $(which unzip) == "" ]]
+  if [[ $(which unzip) = "" ]]
   then echo "Error: Install 'unzip' command in your system";
        exit 1;
   fi 
 }
 
-function translate_odt
+translate_odt ()
 {
   INPUT_TMPDIR=/tmp/$$odtdir
 
   locale_utf8
   test_zip
 
-  if [[ $FICHERO == "" ]]
+  if [[ $FICHERO = "" ]]
   then FICHERO=/tmp/$$odtorig
        cat > $FICHERO
        BORRAFICHERO="true"
@@ -54,43 +58,52 @@ function translate_odt
   find $INPUT_TMPDIR | grep content\\\.xml |\
   awk '{printf "<file name=\"" $0 "\"/>"; PART = $0; while(getline < PART) printf(" %s", $0); printf("\n");}' |\
   $APERTIUM_PATH/apertium-desodt |\
+  if [ "$TRANSLATION_MEMORY_FILE" = "" ]; 
+  then cat;  
+  else $APERTIUM_PATH/lt-tmxproc $TMCOMPFILE;
+  fi | \
   if [ ! -x $DATOS/modes/$PREFIJO.mode ]
-  then sh $DATOS/modes/$PREFIJO.mode $OPTION
-  else $DATOS/modes/$PREFIJO.mode $OPTION
+  then sh $DATOS/modes/$PREFIJO.mode $OPTION $OPTION_TAGGER
+  else $DATOS/modes/$PREFIJO.mode $OPTION $OPTION_TAGGER
   fi | \
   $APERTIUM_PATH/apertium-reodt|\
-  awk '{punto = index($0, "<?"); cabeza = substr($0, 1, punto-1); cola = substr($0, punto); n1 = substr(cabeza, index(cabeza, "\"")+1); name = substr(n1, 1, index(n1, "\"")-1); gsub("\?> ", "?>\n", cola); print cola > name;}'
+  awk '{punto = index($0, "<?"); cabeza = substr($0, 1, punto-1); cola = substr($0, punto); n1 = substr(cabeza, index(cabeza, "\"")+1); name = substr(n1, 1, index(n1, "\"")-1); gsub("[?]> ", "?>\n", cola); print cola > name;}'
   VUELVE=$(pwd)
   cd $INPUT_TMPDIR
+  rm -Rf ObjectReplacements
   zip -q -r $OTRASALIDA .
   cd $VUELVE
   rm -Rf $INPUT_TMPDIR
 
-  if [[ $BORRAFICHERO == "true" ]]
+  if [[ $BORRAFICHERO = "true" ]]
   then rm -Rf $FICHERO;
   fi
-  if [[ $SALIDA == "" ]]
+  if [[ $SALIDA = "" ]]
   then cat $OTRASALIDA;
        rm -Rf $OTRASALIDA
   else mv $OTRASALIDA $SALIDA
   fi
+  
+  if [ "$TRANSLATION_MEMORY_FILE" != "" ];
+  then rm -Rf $TMCOMPFILE
+  fi
 }
 
-function translate_docx
+translate_docx ()
 {
   INPUT_TMPDIR=/tmp/$$docxdir
 
   locale_utf8
   test_zip
   
-  if [[ $FICHERO == "" ]]
+  if [[ $FICHERO = "" ]]
   then FICHERO=/tmp/$$docxorig
        cat > $FICHERO
        BORRAFICHERO="true"
   fi
   OTRASALIDA=/tmp/$$docxsalida.zip
   
-  if [[ $UWORDS == "no" ]]
+  if [[ $UWORDS = "no" ]]
   then OPCIONU="-u";
   else OPCIONU="";
   fi
@@ -106,36 +119,104 @@ function translate_docx
   grep -v -i \\\(settings\\\|theme\\\|styles\\\|font\\\|rels\\\) |\
   awk '{printf "<file name=\"" $0 "\"/>"; PART = $0; while(getline < PART) printf(" %s", $0); printf("\n");}' |\
   $APERTIUM_PATH/apertium-deswxml |\
+  if [ "$TRANSLATION_MEMORY_FILE" = "" ]; 
+  then cat;  
+  else $APERTIUM_PATH/lt-tmxproc $TMCOMPFILE;
+  fi | \
   if [ ! -x $DATOS/modes/$PREFIJO.mode ]
-  then sh $DATOS/modes/$PREFIJO.mode $OPTION
-  else $DATOS/modes/$PREFIJO.mode $OPTION
+  then sh $DATOS/modes/$PREFIJO.mode $OPTION $OPTION_TAGGER
+  else $DATOS/modes/$PREFIJO.mode $OPTION $OPTION_TAGGER
   fi | \
   $APERTIUM_PATH/apertium-rewxml|\
-  awk '{punto = index($0, "<?"); cabeza = substr($0, 1, punto-1); cola = substr($0, punto); n1 = substr(cabeza, index(cabeza, "\"")+1); name = substr(n1, 1, index(n1, "\"")-1); gsub("\?> ", "?>\n", cola); print cola > name;}'
+  awk '{punto = index($0, "<?"); cabeza = substr($0, 1, punto-1); cola = substr($0, punto); n1 = substr(cabeza, index(cabeza, "\"")+1); name = substr(n1, 1, index(n1, "\"")-1); gsub("[?]> ", "?>\n", cola); print cola > name;}'
   VUELVE=$(pwd)
   cd $INPUT_TMPDIR
   zip -q -r $OTRASALIDA .
   cd $VUELVE
   rm -Rf $INPUT_TMPDIR
 
-  if [[ $BORRAFICHERO == "true" ]]
+  if [[ $BORRAFICHERO = "true" ]]
   then rm -Rf $FICHERO;
   fi
-  if [[ $SALIDA == "" ]]
+  if [[ $SALIDA = "" ]]
   then cat $OTRASALIDA;
        rm -Rf $OTRASALIDA
   else mv $OTRASALIDA $SALIDA
   fi
+  if [ "$TRANSLATION_MEMORY_FILE" != "" ];
+  then rm -Rf $TMCOMPFILE
+  fi
 }
 
-function translate_xlsx
+translate_pptx ()
+{
+  INPUT_TMPDIR=/tmp/$$docxdir
+
+  locale_utf8
+  test_zip
+  
+  if [[ $FICHERO = "" ]]
+  then FICHERO=/tmp/$$docxorig
+       cat > $FICHERO
+       BORRAFICHERO="true"
+  fi
+  OTRASALIDA=/tmp/$$docxsalida.zip
+  
+  if [[ $UWORDS = "no" ]]
+  then OPCIONU="-u";
+  else OPCIONU="";
+  fi
+  
+  unzip -q -o -d $INPUT_TMPDIR $FICHERO
+  
+  for i in $(find $INPUT_TMPDIR|grep "xlsx$");
+  do $APERTIUM_PATH/apertium -f xlsx -d $DIRECTORY $OPCIONU $PREFIJO <$i >/tmp/$$xlsxembed;
+     mv /tmp/$$xlsxembed $i;
+  done;
+  
+  find $INPUT_TMPDIR | grep "xml$" |\
+  grep "slides\/slide" |\
+  awk '{printf "<file name=\"" $0 "\"/>"; PART = $0; while(getline < PART) printf(" %s", $0); printf("\n");}' |\
+  $APERTIUM_PATH/apertium-despptx |\
+  if [ "$TRANSLATION_MEMORY_FILE" = "" ]; 
+  then cat;  
+  else $APERTIUM_PATH/lt-tmxproc $TMCOMPFILE;
+  fi | \
+  if [ ! -x $DATOS/modes/$PREFIJO.mode ]
+  then sh $DATOS/modes/$PREFIJO.mode $OPTION $OPTION_TAGGER
+  else $DATOS/modes/$PREFIJO.mode $OPTION $OPTION_TAGGER
+  fi | \
+  $APERTIUM_PATH/apertium-repptx |\
+  awk '{punto = index($0, "<?"); cabeza = substr($0, 1, punto-1); cola = substr($0, punto); n1 = substr(cabeza, index(cabeza, "\"")+1); name = substr(n1, 1, index(n1, "\"")-1); gsub("[?]> ", "?>\n", cola); print cola > name;}'
+  VUELVE=$(pwd)
+  cd $INPUT_TMPDIR
+  zip -q -r $OTRASALIDA .
+  cd $VUELVE
+  rm -Rf $INPUT_TMPDIR
+
+  if [[ $BORRAFICHERO = "true" ]]
+  then rm -Rf $FICHERO;
+  fi
+  if [[ $SALIDA = "" ]]
+  then cat $OTRASALIDA;
+       rm -Rf $OTRASALIDA
+  else mv $OTRASALIDA $SALIDA
+  fi
+  
+  if [ "$TRANSLATION_MEMORY_FILE" != "" ];
+  then rm -Rf $TMCOMPFILE
+  fi
+}
+
+
+translate_xlsx ()
 {
   INPUT_TMPDIR=/tmp/$$xlsxdir
 
   locale_utf8
   test_zip
   
-  if [[ $FICHERO == "" ]]
+  if [[ $FICHERO = "" ]]
   then FICHERO=/tmp/$$xlsxorig
        cat > $FICHERO
        BORRAFICHERO="true"
@@ -146,37 +227,48 @@ function translate_xlsx
   find $INPUT_TMPDIR | grep "sharedStrings.xml" |\
   awk '{printf "<file name=\"" $0 "\"/>"; PART = $0; while(getline < PART) printf(" %s", $0); printf("\n");}' |\
   $APERTIUM_PATH/apertium-desxlsx |\
-  if [ ! -x $DATOS/modes/$PREFIJO.mode ]
-  then sh $DATOS/modes/$PREFIJO.mode $OPTION
-  else $DATOS/modes/$PREFIJO.mode $OPTION
+  if [ "$TRANSLATION_MEMORY_FILE" = "" ]; 
+  then cat;  
+  else $APERTIUM_PATH/lt-tmxproc $TMCOMPFILE;
   fi | \
-  $APERTIUM_PATH/apertium-rexlsx|\
-  awk '{punto = index($0, "<?"); cabeza = substr($0, 1, punto-1); cola = substr($0, punto); n1 = substr(cabeza, index(cabeza, "\"")+1); name = substr(n1, 1, index(n1, "\"")-1); gsub("\?> ", "?>\n", cola); print cola > name;}'
+  if [ ! -x $DATOS/modes/$PREFIJO.mode ]
+  then sh $DATOS/modes/$PREFIJO.mode $OPTION $OPTION_TAGGER
+  else $DATOS/modes/$PREFIJO.mode $OPTION $OPTION_TAGGER
+  fi | \
+  $APERTIUM_PATH/apertium-rexlsx |\
+  awk '{punto = index($0, "<?"); cabeza = substr($0, 1, punto-1); cola = substr($0, punto); n1 = substr(cabeza, index(cabeza, "\"")+1); name = substr(n1, 1, index(n1, "\"")-1); gsub("[?]> ", "?>\n", cola); print cola > name;}'
   VUELVE=$(pwd)
   cd $INPUT_TMPDIR
   zip -q -r $OTRASALIDA .
   cd $VUELVE
   rm -Rf $INPUT_TMPDIR
 
-  if [[ $BORRAFICHERO == "true" ]]
+  if [[ $BORRAFICHERO = "true" ]]
   then rm -Rf $FICHERO;
   fi
-  if [[ $SALIDA == "" ]]
+  if [[ $SALIDA = "" ]]
   then cat $OTRASALIDA;
        rm -Rf $OTRASALIDA
   else mv $OTRASALIDA $SALIDA
   fi
+  
+  if [ "$TRANSLATION_MEMORY_FILE" != "" ];
+  then rm -Rf $TMCOMPFILE
+  fi
 }
 
 
-ARGS=$(getopt "uhf:d:" $*)
+ARGS=$(getopt "uahf:d:m:o:" $*)
 set -- $ARGS
 for i
 do
   case "$i" in 
     -f) shift; FORMAT=$1; shift;;
     -d) shift; DIRECTORY=$1; shift;;
+    -m) shift; TRANSLATION_MEMORY_FILE=$1; shift;;
+    -o) shift; TRANSLATION_MEMORY_DIRECTION=$1 shift;;
     -u) UWORDS="no"; shift;;
+    -a) OPTION_TAGGER="-m"; shift;;
     -h) message;;
     --) shift; break;;
   esac
@@ -210,12 +302,23 @@ esac
 
 if [ x$FORMAT = x ]; then FORMAT="txt"; fi
 if [ x$DIRECTORY = x ]; then DIRECTORY=$DEFAULT_DIRECTORY; fi
+if [ x$TRANSLATION_MEMORY_DIRECTION = x ]; then TRANSLATION_MEMORY_DIRECTION=$PAIR; fi
 
 PREFIJO=$PAIR;
 FORMATADOR=$FORMAT;
 DATOS=$DIRECTORY;
 FICHERO=$INPUT_FILE;
 SALIDA=$OUTPUT_FILE;
+TMCOMPFILE="/tmp/$$tm"
+
+if [ "$TRANSLATION_MEMORY_FILE" != "" ]
+then $APERTIUM_PATH/lt-tmxcomp $TRANSLATION_MEMORY_DIRECTION $TRANSLATION_MEMORY_FILE $TMCOMPFILE >/dev/null
+     if [ "$?" != "0" ]
+     then echo "Error: Cannot compile TM '" $TRANSLATION_MEMORY_FILE "'";
+          echo"   hint: use -o parameter";
+          message;
+     fi
+fi
 
 if [ ! -d $DATOS/modes ];
 then echo "Error: Directory '$DATOS/modes' does not exist."
@@ -239,16 +342,16 @@ fi
 
 case "$FORMATADOR" in 
 	txt|rtf|html)
-		if [[ $UWORDS == "no" ]]; then OPTION="-n"; 
+		if [[ $UWORDS = "no" ]]; then OPTION="-n"; 
 		else OPTION="-g";
 		fi;
 		;;
         rtf)
-		if [[ $UWORDS == "no" ]]; then OPTION="-n"; 
+		if [[ $UWORDS = "no" ]]; then OPTION="-n"; 
 		else OPTION="-g";
 		fi;
 		MILOCALE=$(locale -a|grep -i -v "utf\|^C$\|^POSIX$"|head -1);
-		if [[ $MILOCALE == "" ]]
+		if [[ $MILOCALE = "" ]]
 		then echo "Error: Install a ISO-8859-1 compatible locale in your system";
 	             exit 1;
 	        fi
@@ -256,30 +359,36 @@ case "$FORMATADOR" in
 		;;
         
         odt)
-		if [[ $UWORDS == "no" ]]; then OPTION="-n"; 
+		if [[ $UWORDS = "no" ]]; then OPTION="-n"; 
 		else OPTION="-g";
 		fi;
 		translate_odt
 		exit 0
 		;;
 	docx)
-		if [[ $UWORDS == "no" ]]; then OPTION="-n"; 
+		if [[ $UWORDS = "no" ]]; then OPTION="-n"; 
 		else OPTION="-g";
 		fi;
 		translate_docx
 		exit 0
 		;;
 	xlsx)
-		if [[ $UWORDS == "no" ]]; then OPTION="-n"; 
+		if [[ $UWORDS = "no" ]]; then OPTION="-n"; 
 		else OPTION="-g";
 		fi;
 		translate_xlsx
 		exit 0
 		;;
-		
+	pptx)
+		if [[ $UWORDS = "no" ]]; then OPTION="-n"; 
+		else OPTION="-g";
+		fi;
+		translate_pptx
+		exit 0
+		;;
 		
 	wxml)
-		if [[ $UWORDS == "no" ]]; then OPTION="-n";
+		if [[ $UWORDS = "no" ]]; then OPTION="-n";
 		else OPTION="-g";
 		fi;
 	        locale_utf8
@@ -297,7 +406,7 @@ case "$FORMATADOR" in
 		FORMATADOR="rtf";
 		OPTION="-n";
 		MILOCALE=$(locale -a|grep -i -v "utf\|^C$\|^POSIX$"|head -1);
-		if [[ $MILOCALE == "" ]]
+		if [[ $MILOCALE = "" ]]
 		then echo "Error: Install a ISO-8859-1 compatible locale in your system";
 	             exit 1;
 	        fi
@@ -321,7 +430,12 @@ case "$FORMATADOR" in
                 translate_xlsx
                 exit 0
                 ;;
-        
+
+        pptxu)
+                OPTION="-n"
+                translate_pptx
+                exit 0
+                ;;
         
         wxmlu)
 		OPTION="-n";
@@ -341,13 +455,23 @@ then
         REF=$FORMATADOR
 fi
 
+
+
 $APERTIUM_PATH/apertium-des$FORMATADOR $FICHERO | \
+if [ "$TRANSLATION_MEMORY_FILE" = "" ]; 
+then cat;  
+else $APERTIUM_PATH/lt-tmxproc $TMCOMPFILE;
+fi | \
 if [ ! -x $DATOS/modes/$PREFIJO.mode ]
-then sh $DATOS/modes/$PREFIJO.mode $OPTION
-else $DATOS/modes/$PREFIJO.mode $OPTION
+then sh $DATOS/modes/$PREFIJO.mode $OPTION $OPTION_TAGGER
+else $DATOS/modes/$PREFIJO.mode $OPTION $OPTION_TAGGER
 fi | \
 if [ x$SALIDA = x ]
 then $APERTIUM_PATH/apertium-re$FORMATADOR 
 else
   $APERTIUM_PATH/apertium-re$FORMATADOR >$SALIDA
 fi
+
+if [ "$TRANSLATION_MEMORY_FILE" != "" ];
+then rm -Rf $TMCOMPFILE
+fi
