@@ -39,22 +39,25 @@ void help(char *name) {
   wcerr<<name<<L" --mono dic.bin --bil bildic.bin --wlist wlistfile\n\n";
   wcerr<<L"ARGUMENTS: \n"
       <<L"   --mono|-m: Specifies the monolingual lexical selection dictionary to use.\n"
-      <<L"   --bil|-b: Specifies the bilingual lexical selection ditionary to use.\n"
+      <<L"   --bil|-b: Specifies the bilingual lexical selection dictionary to use.\n"
       <<L"   --wlist|-w: Specifies the list of words to translate.\n"
       <<L"   --help|-h: Show this help\n"
       <<L"   --version|-v: Show version information\n\n"
-      <<L"Write to stadard output all possible translations of words found in wlistfile\n";
+      <<L"Write to standard output all possible translations of words found in wlistfile\n";
 }
 
 int main(int argc, char* argv[]) {
   int c;
+  
+#if HAVE_GETOPT_LONG
   int option_index=0;
-
+#endif
   string monodic_file="";
   string bildic_file="";
   string wlist_file="";
 
   while (true) {
+#if HAVE_GETOPT_LONG
     static struct option long_options[] =
       {
 	{"mono",    required_argument, 0, 'm'},
@@ -66,6 +69,9 @@ int main(int argc, char* argv[]) {
       };
 
     c=getopt_long(argc, argv, "m:b:w:hv",long_options, &option_index);
+#else
+    c=getopt(argc, argv, "m:b:w:hv");
+#endif
     if (c==-1)
       break;
       
