@@ -1,3 +1,4 @@
+
 /*
  * Copyright (C) 2005 Universitat d'Alacant / Universidad de Alicante
  *
@@ -24,7 +25,13 @@
 
 #include <apertium/hmm.h>
 #include <apertium/tagger_utils.h>
-#include <lttoolbox/endian_util.h>
+#include <apertium/unlocked_cstdio.h>
+#include <lttoolbox/compression.h>
+
+#ifdef WIN32
+#define isnan(n) _isnan(n)
+#define isinf(n) (!_finite(n))
+#endif
 
 #include <stdio.h>
 #include <unistd.h>
@@ -40,6 +47,7 @@ HMM::HMM(TaggerData *t)
   this->td = t;
 
   debug=false;
+  show_sf=false;
   eos = (td->getTagIndex())[L"TAG_SENT"];  
 }
 
@@ -64,12 +72,19 @@ HMM::set_debug(bool d)
   debug = d; 
 } 
 
+void
+HMM::set_show_sf(bool sf)
+{ 
+  show_sf = sf; 
+} 
+
+
 void 
 HMM::read_ambiguity_classes(FILE *in) 
 {
   while(in)
   {
-    int ntags = EndianUtil<int>::read(in);
+    int ntags = Compression::multibyte_read(in);
 
     if(feof(in))
     {
@@ -79,7 +94,7 @@ HMM::read_ambiguity_classes(FILE *in)
 
     for(; ntags != 0; ntags--)
     {
-      ambiguity_class.insert(EndianUtil<TTag>::read(in));
+      ambiguity_class.insert(Compression::multibyte_read(in));
     }
     
     if(ambiguity_class.size() != 0)
@@ -97,11 +112,11 @@ HMM::write_ambiguity_classes(FILE *out)
   for(int i=0, limit = td->getOutput().size(); i != limit; i++) 
   {
     set<TTag> const &ac = (td->getOutput())[i];
-    EndianUtil<int>::write(out, ac.size());
+    Compression::multibyte_write(ac.size(), out);
     for(set<TTag>::const_iterator it = ac.begin(), limit2 = ac.end();
         it != limit2; it++)
     {
-      EndianUtil<TTag>::write(out, *it);
+      Compression::multibyte_write(*it, out);
     }
   } 
 }  
@@ -786,9 +801,11 @@ HMM::tagger(FILE *in, FILE *out, bool show_all_good_first) {
       }
       for (unsigned t=0; t<best[nwpend%2][tag].size(); t++) {
 	if (show_all_good_first) {
-	  wstring const &micad = wpend[t].get_all_choosen_tag_first(best[nwpend%2][tag][t], (td->getTagIndex())[L"TAG_kEOF"]);
+	  wstring const &micad = wpend[t].get_all_chosen_tag_first(best[nwpend%2][tag][t], (td->getTagIndex())[L"TAG_kEOF"]);
 	  fputws_unlocked(micad.c_str(), out); 
 	} else {
+	  // print out
+	  wpend[t].set_show_sf(show_sf);
 	  wstring const &micad = wpend[t].get_lexical_form(best[nwpend%2][tag][t], (td->getTagIndex())[L"TAG_kEOF"]);
 	  fputws_unlocked(micad.c_str(), out); 
 	}
