@@ -54,7 +54,8 @@ private:
    TaggerData *td;
    TTag eos; // end-of-sentence tag
    bool debug;  //If true, print error messages when tagging input text
-
+   bool show_sf;  //If true, print superficial forms when tagging input text
+   
    /** It allocs memory for the transition (a) and the emission (b) matrices.
     *  Before calling this method the number of ambiguity classes must be known.
     *  This methos is called within read_ambiguity_classes and read_dictionary.
@@ -91,6 +92,11 @@ public:
     */
    void set_debug(bool d);
 
+   /** Used to set the show superficial forms flag 
+    *
+    */
+   void set_show_sf(bool sf);
+
    /** It reads the ambiguity classes from the stream received as
     *  input
     *  @param is the input stream
