@@ -26,6 +26,7 @@
 #include <iostream>
 #include <stack>
 #include <apertium/string_utils.h>
+#include <apertium/unlocked_cstdio.h>
 
 using namespace Apertium;
 using namespace std;
@@ -598,6 +599,7 @@ Interchunk::processCallMacro(xmlNode *localroot)
   }
 
   int idx = 0;
+  int lastpos = 0;
   for(xmlNode *i = localroot->children; i != NULL; i = i->next)
   {
     if(i->type == XML_ELEMENT_NODE)
@@ -606,14 +608,16 @@ Interchunk::processCallMacro(xmlNode *localroot)
       myword[idx] = word[pos];
       if(idx-1 >= 0)
       {
-        myblank[idx-1] = blank[pos-1];
+        myblank[idx-1] = blank[lastpos];
       }
       idx++;
+      lastpos = pos;
     }
   }
 
   swap(myword, word);
   swap(myblank, blank);
+  swap(npar, lword);
   
   for(xmlNode *i = macro->children; i != NULL; i = i->next)
   {
@@ -625,6 +629,7 @@ Interchunk::processCallMacro(xmlNode *localroot)
 
   swap(myword, word);
   swap(myblank, blank);
+  swap(npar, lword);
 
   if(myword)
   {
@@ -694,15 +699,23 @@ Interchunk::processLogical(xmlNode *localroot)
   {
     return processEqual(localroot);
   }
-  if(!xmlStrcmp(localroot->name, (const xmlChar *) "begins-with"))
+  else if(!xmlStrcmp(localroot->name, (const xmlChar *) "begins-with"))
   {
     return processBeginsWith(localroot);
   }
-  if(!xmlStrcmp(localroot->name, (const xmlChar *) "ends-with"))
+  else if(!xmlStrcmp(localroot->name, (const xmlChar *) "begins-with-list"))
+  {
+    return processBeginsWithList(localroot);
+  }
+  else if(!xmlStrcmp(localroot->name, (const xmlChar *) "ends-with"))
   {
     return processEndsWith(localroot);
   }
-  if(!xmlStrcmp(localroot->name, (const xmlChar *) "contains-substring"))
+  else if(!xmlStrcmp(localroot->name, (const xmlChar *) "ends-with-list"))
+  {
+    return processEndsWithList(localroot);
+  }
+  else if(!xmlStrcmp(localroot->name, (const xmlChar *) "contains-substring"))
   {
     return processContainsSubstring(localroot);
   }
@@ -902,9 +915,9 @@ Interchunk::endsWith(string const &s1, string const &s2) const
   {
     return false;
   }
-  for(int i = limit-1; i >= 0; i++)
+  for(int i = limit-1, j = constraint - 1; i >= 0; i--, j--)
   {
-    if(s1[i] != s2[i])
+    if(s1[j] != s2[i])
     {
       return false;
     }
@@ -992,6 +1005,102 @@ Interchunk::processEndsWith(xmlNode *localroot)
   }
 }
 
+bool
+Interchunk::processBeginsWithList(xmlNode *localroot)
+{
+  xmlNode *first = NULL, *second = NULL;
+
+  for(xmlNode *i = localroot->children; i != NULL; i = i->next)
+  {
+    if(i->type == XML_ELEMENT_NODE)
+    {
+      if(first == NULL)
+      {
+        first = i;
+      }
+      else
+      {
+	second = i;
+	break;
+      }
+    }
+  }
+
+  xmlChar *idlist = second->properties->children->content;
+  string needle = evalString(first);
+  set<string, Ltstr>::iterator it, limit;
+
+  if(localroot->properties == NULL || 
+     xmlStrcmp(localroot->properties->children->content, (const xmlChar *) "yes"))
+  {
+    it = lists[(const char *) idlist].begin();
+    limit = lists[(const char *) idlist].end();
+  }
+  else
+  {
+    needle = tolower(needle);
+    it = listslow[(const char *) idlist].begin();
+    limit = listslow[(const char *) idlist].end();
+  }
+  
+  for(; it != limit; it++)
+  {
+    if(beginsWith(needle, *it))
+    {
+      return true;
+    }
+  }
+  return false;
+}
+
+bool
+Interchunk::processEndsWithList(xmlNode *localroot)
+{
+  xmlNode *first = NULL, *second = NULL;
+
+  for(xmlNode *i = localroot->children; i != NULL; i = i->next)
+  {
+    if(i->type == XML_ELEMENT_NODE)
+    {
+      if(first == NULL)
+      {
+        first = i;
+      }
+      else
+      {
+	second = i;
+	break;
+      }
+    }
+  }
+
+  xmlChar *idlist = second->properties->children->content;
+  string needle = evalString(first);
+  set<string, Ltstr>::iterator it, limit;
+
+  if(localroot->properties == NULL || 
+     xmlStrcmp(localroot->properties->children->content, (const xmlChar *) "yes"))
+  {
+    it = lists[(const char *) idlist].begin();
+    limit = lists[(const char *) idlist].end();
+  }
+  else
+  {
+    needle = tolower(needle);
+    it = listslow[(const char *) idlist].begin();
+    limit = listslow[(const char *) idlist].end();
+  }
+  
+  for(; it != limit; it++)
+  {
+    if(endsWith(needle, *it))
+    {
+      return true;
+    }
+  }
+  return false;
+}
+
 bool
 Interchunk::processContainsSubstring(xmlNode *localroot)
 {
