@@ -26,6 +26,7 @@
 #include <apertium/hmm.h>
 #include <apertium/tagger_utils.h>
 #include <apertium/tsx_reader.h>
+#include <apertium/tagger_word.h>
 
 #include <cstdio>
 #include <fstream>
@@ -37,38 +38,67 @@
 #include <unistd.h>
 #include <getopt.h>
 #include <apertium/string_utils.h>
+#ifdef WIN32
+#include <io.h>
+#include <fcntl.h>
+#endif
 
 using namespace Apertium;
 using namespace std;
 
+void
+Tagger::setShowSF(bool val) 
+{
+  showSF = val;
+}
+
+bool
+Tagger::getShowSF()
+{
+  return showSF;
+}
+
 int
 Tagger::getMode(int argc, char *argv[])
 {
   int mode=UNKNOWN_MODE;
    
   int c;
+
+#if HAVE_GETOPT_LONG
   int option_index=0;
+#endif
 
   while (true) {
+#if HAVE_GETOPT_LONG
     static struct option long_options[] =
     {
       {"train",      required_argument, 0, 't'},
       {"supervised", required_argument, 0, 's'},
       {"retrain",    required_argument, 0, 'r'},
       {"tagger",     no_argument,       0, 'g'},
+      {"show-superficial",     no_argument,       0, 'p'},
       {"eval",       no_argument,       0, 'e'},
       {"first",      no_argument,       0, 'f'},
       {"help",       no_argument,       0, 'h'}, 
       {"debug",      no_argument,       0, 'd'}, 
+      {"mark",       no_argument,       0, 'm'},
       {0, 0, 0, 0}
     };
 
-    c=getopt_long(argc, argv, "dt:s:r:gefh",long_options, &option_index);
+    c=getopt_long(argc, argv, "mdt:s:r:gpefh",long_options, &option_index);
+#else
+    c=getopt(argc, argv, "mdt:s:r:gpefh");
+#endif
     if (c==-1)
       break;
       
     switch (c)
     {
+      case 'm':
+	TaggerWord::generate_marks = true;
+        break;
+        
       case 'd':
         debug=true;
         break;
@@ -115,6 +145,10 @@ Tagger::getMode(int argc, char *argv[])
 	  help();
 	}
 	break;
+
+      case 'p':
+        setShowSF(true);
+	break;
 	
       case 'r':
         if(!isNumber(optarg))
@@ -242,6 +276,7 @@ Tagger::getMode(int argc, char *argv[])
 Tagger::Tagger()
 {
   debug = false;
+  showSF = false;
 }
 
 void
@@ -292,7 +327,9 @@ Tagger::tagger(bool mode_first)
   fclose(ftdata);
   
   HMM hmm(&td);
-  
+
+  hmm.set_show_sf(showSF);
+
   if(filenames.size() == 1)
   {
     hmm.tagger(stdin, stdout, mode_first);
@@ -303,6 +340,9 @@ Tagger::tagger(bool mode_first)
     if (!finput) {
       filerror(filenames[1]);
     }
+#ifdef WIN32
+	_setmode(_fileno(finput), _O_U8TEXT);
+#endif
     if(filenames.size() == 2)
     {
       hmm.tagger(finput, stdout, mode_first);
@@ -313,6 +353,9 @@ Tagger::tagger(bool mode_first)
       if (!foutput) {
         filerror(filenames[2]);
       }
+#ifdef WIN32
+	  _setmode(_fileno(foutput), _O_U8TEXT);
+#endif
 
       hmm.tagger(finput, foutput, mode_first);
       fclose(foutput);
@@ -352,6 +395,9 @@ Tagger::train()
   FILE *fcrp = fopen(filenames[1].c_str(), "r");
   if(fcrp)
   {
+#ifdef WIN32
+    _setmode(_fileno(fcrp), _O_U8TEXT);
+#endif 
     hmm.init_probabilities_kupiec(fcrp);               
   }
   else
@@ -401,6 +447,10 @@ Tagger::trainSupervised()
   FILE *funtagged = fopen(filenames[5].c_str(), "r");
   if(ftagged && funtagged)
   {
+#ifdef WIN32
+    _setmode(_fileno(ftagged), _O_U8TEXT);
+    _setmode(_fileno(funtagged), _O_U8TEXT);
+#endif 
     wcerr << L"Initializing transition and emission probabilities from a hand-tagged corpus...\n";
     hmm.init_probabilities_from_tagged_text(ftagged, funtagged);
   }
@@ -418,6 +468,9 @@ Tagger::trainSupervised()
   FILE *fcrp = fopen(filenames[1].c_str(), "r");
   if(fcrp)
   {
+#ifdef WIN32
+    _setmode(_fileno(fcrp), _O_U8TEXT);
+#endif 
     for(int i=0; i != nit; i++)
     {
       fseek(fcrp, 0, SEEK_SET);
@@ -458,6 +511,9 @@ Tagger::retrain()
   {
     filerror(filenames[0]);
   }
+#ifdef WIN32
+  _setmode(_fileno(fcrp), _O_U8TEXT);
+#endif 
   wcerr << L"Training (Baum-Welch)...\n";
   for(int i=0; i != nit; i++)
   {
@@ -481,7 +537,7 @@ void
 Tagger::help()
 {
   ostream &out = cerr;
-  char localname[name.size()+1];
+  char* localname=new char[name.size()+1];
   strcpy(localname, name.c_str());
   out << basename(localname) << ": HMM part-of-speech tagging and training program" << endl;
   out << "GENERIC USAGE: " << basename(localname) << "[-d] <OPTION>=[PARAM] [FILES]" << endl;
@@ -498,10 +554,13 @@ Tagger::help()
   out << "  -r, --retrain=n:     retrains the model with n aditional Baum-Welch" << endl;
   out << "                       iterations (unsupervised)" << endl;
   out << "  -g, --tagger:        tags input text by means of Viterbi algorithm" << endl;
+  out << "  -p, --show-superficial: " << endl;
+  out << "                       show superficial forms in the output stream" << endl;
   out << "  -f, --first:         used if conjuntion with -g (--tagger) makes the tagger"<< endl;
-  out << "                       to give all lexical forms of each word, being the choosen" << endl;
+  out << "                       to give all lexical forms of each word, being the chosen" << endl;
   out << "                       one in the first place (after the lemma)"<<endl;
   out << "  -d, --debug:         print error mesages when tagging input text" << endl;
+  out << "  -m, --mark:          generate marks of solved ambiguities" << endl;
   out << endl;
   out << "And FILES are:" << endl;          
   out << "  DIC:         full expanded dictionary file" << endl;
@@ -514,6 +573,7 @@ Tagger::help()
   out << "               corpus to use both jointly with -s option" << endl; 
   out << "  INPUT:       input file, stdin by default" << endl;
   out << "  OUTPUT:      output file, stdout by default" << endl;
+  delete[] localname;
   exit(EXIT_FAILURE);
 }
 
