@@ -57,6 +57,8 @@ Transfer::Transfer()
   root_element = NULL;
   lastrule = NULL;
   defaultAttrs = lu;
+  useBilingual = true;
+  isExtended = false;
 }
 
 Transfer::~Transfer()
@@ -150,6 +152,21 @@ Transfer::readBil(string const &fstfile)
   fclose(in);
 }
 
+void
+Transfer::setExtendedDictionary(string const &fstfile)
+{
+  FILE *in = fopen(fstfile.c_str(), "rb");
+  if(!in)
+  {
+    cerr << "Error: Could not open extended dictionary file '" << fstfile << "'." << endl;
+    exit(EXIT_FAILURE);
+  }
+  extended.load(in);
+  extended.initBiltrans();
+  fclose(in);
+  isExtended = true;
+}
+
 void
 Transfer::read(string const &transferfile, string const &datafile,
 	       string const &fstfile)
@@ -166,7 +183,10 @@ Transfer::read(string const &transferfile, string const &datafile,
   readData(in);
   fclose(in);
   
-  readBil(fstfile);
+  if(fstfile != "")
+  {
+    readBil(fstfile);
+  }
 }
 
 void
@@ -981,6 +1001,7 @@ Transfer::processCallMacro(xmlNode *localroot)
   }
 
   int idx = 0;
+  int lastpos = 0;
   for(xmlNode *i = localroot->children; i != NULL; i = i->next)
   {
     if(i->type == XML_ELEMENT_NODE)
@@ -989,14 +1010,16 @@ Transfer::processCallMacro(xmlNode *localroot)
       myword[idx] = word[pos];
       if(idx-1 >= 0)
       {
-        myblank[idx-1] = blank[pos-1];
+        myblank[idx-1] = blank[lastpos];
       }
       idx++;
+      lastpos = pos;
     }
   }
 
   swap(myword, word);
   swap(myblank, blank);
+  swap(npar, lword);
   
   for(xmlNode *i = macro->children; i != NULL; i = i->next)
   {
@@ -1008,7 +1031,8 @@ Transfer::processCallMacro(xmlNode *localroot)
 
   swap(myword, word);
   swap(myblank, blank);
-
+  swap(npar, lword);
+  
   if(myword)
   {
     delete myword;
@@ -1077,15 +1101,23 @@ Transfer::processLogical(xmlNode *localroot)
   {
     return processEqual(localroot);
   }
-  if(!xmlStrcmp(localroot->name, (const xmlChar *) "begins-with"))
+  else if(!xmlStrcmp(localroot->name, (const xmlChar *) "begins-with"))
   {
     return processBeginsWith(localroot);
   }
-  if(!xmlStrcmp(localroot->name, (const xmlChar *) "ends-with"))
+  else if(!xmlStrcmp(localroot->name, (const xmlChar *) "begins-with-list"))
+  {
+    return processBeginsWithList(localroot);
+  }
+  else if(!xmlStrcmp(localroot->name, (const xmlChar *) "ends-with"))
   {
     return processEndsWith(localroot);
   }
-  if(!xmlStrcmp(localroot->name, (const xmlChar *) "contains-substring"))
+  else if(!xmlStrcmp(localroot->name, (const xmlChar *) "ends-with-list"))
+  {
+    return processEndsWithList(localroot);
+  }
+  else if(!xmlStrcmp(localroot->name, (const xmlChar *) "contains-substring"))
   {
     return processContainsSubstring(localroot);
   }
@@ -1285,9 +1317,9 @@ Transfer::endsWith(string const &s1, string const &s2) const
   {
     return false;
   }
-  for(int i = limit-1; i >= 0; i++)
+  for(int i = limit-1, j = constraint - 1; i >= 0; i--, j--)
   {
-    if(s1[i] != s2[i])
+    if(s1[j] != s2[i])
     {
       return false;
     }
@@ -1375,6 +1407,103 @@ Transfer::processEndsWith(xmlNode *localroot)
   }
 }
 
+bool
+Transfer::processBeginsWithList(xmlNode *localroot)
+{
+  xmlNode *first = NULL, *second = NULL;
+
+  for(xmlNode *i = localroot->children; i != NULL; i = i->next)
+  {
+    if(i->type == XML_ELEMENT_NODE)
+    {
+      if(first == NULL)
+      {
+        first = i;
+      }
+      else
+      {
+	second = i;
+	break;
+      }
+    }
+  }
+
+  xmlChar *idlist = second->properties->children->content;
+  string needle = evalString(first);
+  set<string, Ltstr>::iterator it, limit;
+
+  if(localroot->properties == NULL || 
+     xmlStrcmp(localroot->properties->children->content, (const xmlChar *) "yes"))
+  {
+    it = lists[(const char *) idlist].begin();
+    limit = lists[(const char *) idlist].end();
+  }
+  else
+  {
+    needle = tolower(needle);
+    it = listslow[(const char *) idlist].begin();
+    limit = listslow[(const char *) idlist].end();
+  }
+  
+  for(; it != limit; it++)
+  {
+    if(beginsWith(needle, *it))
+    {
+      return true;
+    }
+  }
+  return false;
+}
+
+
+bool
+Transfer::processEndsWithList(xmlNode *localroot)
+{
+  xmlNode *first = NULL, *second = NULL;
+
+  for(xmlNode *i = localroot->children; i != NULL; i = i->next)
+  {
+    if(i->type == XML_ELEMENT_NODE)
+    {
+      if(first == NULL)
+      {
+        first = i;
+      }
+      else
+      {
+	second = i;
+	break;
+      }
+    }
+  }
+
+  xmlChar *idlist = second->properties->children->content;
+  string needle = evalString(first);
+  set<string, Ltstr>::iterator it, limit;
+
+  if(localroot->properties == NULL || 
+     xmlStrcmp(localroot->properties->children->content, (const xmlChar *) "yes"))
+  {
+    it = lists[(const char *) idlist].begin();
+    limit = lists[(const char *) idlist].end();
+  }
+  else
+  {
+    needle = tolower(needle);
+    it = listslow[(const char *) idlist].begin();
+    limit = listslow[(const char *) idlist].end();
+  }
+  
+  for(; it != limit; it++)
+  {
+    if(endsWith(needle, *it))
+    {
+      return true;
+    }
+  }
+  return false;
+}
+
 bool
 Transfer::processContainsSubstring(xmlNode *localroot)
 {
@@ -1601,7 +1730,31 @@ Transfer::transfer(FILE *in, FILE *out)
       {
 	if(tmpword.size() != 0)
 	{
-	  pair<wstring, int> tr = fstp.biltransWithQueue(*tmpword[0], false);
+	  pair<wstring, int> tr;
+	  if(useBilingual)
+	  {
+	    if(isExtended && (*tmpword[0])[0] == L'*')
+	    {
+	      tr = extended.biltransWithQueue((*tmpword[0]).substr(1), false);
+              if(tr.first[0] == L'@')
+              {
+                tr.first[0] = L'*';
+              }
+              else
+              {
+                tr.first = L"%" + tr.first;
+              }
+            }
+            else
+            {
+	      tr = fstp.biltransWithQueue(*tmpword[0], false);
+            }
+          }
+          else
+          {
+            tr = pair<wstring, int>(*tmpword[0], 0);
+          }
+          
 	  if(tr.first.size() != 0)
 	  {
 	    if(defaultAttrs == lu)
@@ -1614,11 +1767,11 @@ Transfer::transfer(FILE *in, FILE *out)
             {
               if(tr.first[0] == '*')
               {
-                fputws_unlocked(L"^unknown{^", output);
+                fputws_unlocked(L"^unknown<unknown>{^", output);
               }
               else
               {                
-	        fputws_unlocked(L"^default{^", output);
+	        fputws_unlocked(L"^default<default>{^", output);
               }	        
 	      fputws_unlocked(tr.first.c_str(), output);
 	      fputws_unlocked(L"$}$", output);
@@ -1707,7 +1860,15 @@ Transfer::applyRule()
       blank[i-1] = new string(UtfConverter::toUtf8(*tmpblank[i-1]));
     }
     
-    pair<wstring, int> tr = fstp.biltransWithQueue(*tmpword[i], false);
+    pair<wstring, int> tr;
+    if(useBilingual)
+    {
+      tr = fstp.biltransWithQueue(*tmpword[i], false);
+    }
+    else
+    {
+      tr = pair<wstring, int>(*tmpword[i], false);
+    }
 
     word[i] = new TransferWord(UtfConverter::toUtf8(*tmpword[i]), 
 			       UtfConverter::toUtf8(tr.first), tr.second);
@@ -1779,3 +1940,21 @@ Transfer::applyWord(wstring const &word_str)
   }
   ms.step(L'$');
 }
+
+void 
+Transfer::setUseBilingual(bool value) 
+{
+  useBilingual = value;
+}
+
+bool
+Transfer::getUseBilingual(void) const
+{
+  return useBilingual;
+}
+
+void
+Transfer::setCaseSensitiveness(bool value)
+{
+  fstp.setCaseSensitiveMode(value);
+}
\ No newline at end of file
